/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Bookmarks Menu Button.
 *
 * The Initial Developer of the Original Code is Ronny Perinke.
 * Portions created by the Initial Developer are Copyright (C) 2004-2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 *   Ronny Perinke <www.sephiroth-j.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const bmbext = {
	//#######################################
	//########	private functions and properties	########
	//########						########
	//#######################################
	_Prefs:
	{
		hideBM: true,				// value for hiding the bookmarks menu
		hideBMBS: false,				// value for hiding my BMB settings menu entry
		hideDropmarker: false,			// hide the dropmarker?
		MiddleClickOpens: 1,			// middle-click on button loads ...
		RightClickOpens: 2,			// right-click on button loads ...
		ShowElements: 7,			// display optional elements
		firstRun: true,				// is it the first time that the ext is used?
		enableDebug: false			// enable debug output to console and error console
	},
	_MiscVars:
	{
		MenuIsThere: false,			//
		LastOpenFolders: null,			// holds the array for the menus needed to open
		DefaultBMEntryCount: 4,		// the number of menuentries of the default bookmarks-menu without the bookmarks. by default it should be 3 for fx 1.5 and 1.0 (see browser.xul).
		ShortcutKey: null,
		ShortcutKeyPressed: false,		// was the shortcut key pressed?
		PrefsIFace: null,				// refers to the nsIPref interface
		bmbCSSSheet:				// Container for the CSS-Style sheet of BMB
		{
			globalIndex: -1,			// its index in documents.styleSheets
			sheetObj: null,			// the styleSheet object itself
			dropmarkerHidden: false	// did we hide the dropmarker?
		},
	},
	get prefRoot()
	{
		return "BMBExt.Book.";
	},
	get prefRootG()
	{
		return "BMBExt.";
	},
	get StringBundle()
	{
		var stringBundle = document.getElementById("bmbextstr");
		delete this.StringBundle;
		this.StringBundle = stringBundle;
		return stringBundle;
	},
	get PrefsIFace()
	{
		if (!this._MiscVars.PrefsIFace)
		{
			this.PrefsIFace = this.prefRoot;
			this.__defineGetter__("PrefsIFace", function() { return this._Misc.PrefsIFace; });
		}
		return this._MiscVars.PrefsIFace;
	},
	set PrefsIFace(aBranch)
	{
		if (typeof(aBranch) != "string")
		{
			throw Cr.NS_ERROR_ILLEGAL_VALUE + "\nValue must be a string type";
		}
		
		this._MiscVars.PrefsIFace = Cc["@mozilla.org/preferences;1"].getService(Ci.nsIPref);
		this._MiscVars.PrefsIFace = this._MiscVars.PrefsIFace.getBranch(aBranch?aBranch:this.prefRoot);
	},
	get StatusBarObj()
	{
		var statusBar = document.getElementById("statusbar-display");
		delete this.StatusBarObj;
		this.StatusBarObj = statusBar;
		return statusBar;
	},
	get BMBCSSSheet()
	{
		return this._MiscVars.bmbCSSSheet;
	},
	get ConsService()
	{
		var consService = Cc["@mozilla.org/consoleservice;1"].getService(Ci.nsIConsoleService);
		delete this.ConsService;
		this.ConsService = consService;
		return consService;
	},
	dump: function BMB_dump(aMsg, aLine, aType)
	{
		if (!this._Prefs.enableDebug)
		{
			return;
		}
		else
		{
			dump("BookmarkMenuButton::" + aMsg + "\n");
			if (!isNaN(aType) && aType != 3)
			{
				var scriptError = Cc["@mozilla.org/scripterror;1"].createInstance(Ci.nsIScriptError);
				//nsIScriptError.errorFlag = 0, nsIScriptError.warningFlag = 1, nsIScriptError.exceptionFlag = 2 and nsIScriptError.strictFlag = 4.
				scriptError.init("BookmarkMenuButton::" + aMsg, "chrome://bmb-ext/content/bmb-ext.js", null, aLine, 0, aType, "XUL javascript");
				this.ConsService.logMessage(scriptError);
			}
			else
			{
				this.ConsService.logStringMessage("BookmarkMenuButton::" + aMsg);
			}
		}
	},
	//#######################################
	//########	/private functions and properties	########
	//########						########
	//#######################################
	
	/**
		implement EventListener Interface
		only used for addEventListener()
	*/
	handleEvent: function BMB_handleEvent(aEvent)
	{
		switch(aEvent.type)
		{
			case "load":
				this.init();
				break;
			
			case "unload":
				this.quit();
				break;
			
			case "popupshowing":
				if (aEvent.target.id == "menubutton_BookmarksPopup")
				{
					this.UpdateBookmarkAllTabsMenuitem();
				}
				BookmarksEventHandler.onPopupShowing(aEvent);
				break;
			
			case "popuphidden":
				if (aEvent.target.id == "menubutton_BookmarksPopup")
				{
					this.destroyMenu();
				}
				break;
			
			case "keydown":
			case "keyup":
				this.catchKKey(aEvent);
				break;
		}
	},
	
	init: function BMB_init()
	{
		this.patchMethods();
		this.loadGPrefs();
		this.loadPrefs();
		this.deleteOldPrefs();
		this.findBMBCSSSheet();
		this.showHideDropmarker();
		this.showHideOptElements();
		setTimeout("bmbext.addButtonAtFirstTime()", 1000);		//add menubutton to nav-bar on first startup after 1sec
		this.showHide();
		this.setBehavior(false);
		
		var menupopup = document.getElementById("bookmarksMenuPopup");
		try
		{
			this._MiscVars.DefaultBMEntryCount = menupopup.childNodes.length;
			menupopup.lastChild.id = "LastSepBeforeBMs";
		}
		catch (e)
		{
			this._MiscVars.DefaultBMEntryCount = 4;
		}
		
		try
		{
			if (document.getElementById("bookmarks-menu-button"))
			{
				this._MiscVars.ShortcutKey = document.getElementById("bookmarksMenu").getAttribute("accesskey").charCodeAt(0);		//converts the character to the ASCII code because keyCode is a ASCII code
				window.addEventListener("keydown", this, false);
				window.addEventListener("keyup", this, false);
			}
		}
		catch (e)
		{
			alert("BMB Extension\n\nCould not add EventListener #1!\nError Msg is:\n" + e);
		}
	},
	quit: function BMB_quit()
	{
		//clean-up
		this._MiscVars.bmbCSSSheet.sheetObj = null;
		this._MiscVars.PrefsIFace = null;
		this._MiscVars.StatusBarObj = null;
		delete this.ConsService;
		delete this.StringBundle;
	},
	
	deleteOldPrefs: function BMB_deleteOldPrefs()
	{
		if (!this._Prefs.firstRun)
		{
			return;
		}
		this.PrefsIFace = this.prefRootG;
		
		// Added "Book."-Prefix so let's remove the old ones
		try
		{
			this.PrefsIFace.getBoolPref("settings.hide");	//gibt es sie?
			this.PrefsIFace.clearUserPref("settings.hide");	//wenn ja, dann weg damit
		}
		catch (e)
		{
		}
		try
		{
			this.PrefsIFace.getBoolPref("settings.MiddleClickOpens");
			this.PrefsIFace.clearUserPref("settings.MiddleClickOpens");
		}
		catch (e)
		{
		}
		try
		{
			this.PrefsIFace.getBoolPref("settings.RightClickOpens");
			this.PrefsIFace.clearUserPref("settings.RightClickOpens");
		}
		catch (e)
		{
		}
	},
	loadPrefs: function BMB_loadPrefs()
	{
		//load user settings
		this.PrefsIFace = this.prefRoot;	// init PrefsIFace with default pref-branch
		
		this._Prefs.hideBM = this.PrefsIFace.getBoolPref("bookmarkmenu.hide");
		this._Prefs.hideBMBS = this.PrefsIFace.getBoolPref("settings.hide");
		this._Prefs.hideDropmarker = this.PrefsIFace.getBoolPref("dropmarker.hide");
		
		this._Prefs.MiddleClickOpens = this.PrefsIFace.getIntPref("settings.MiddleClickOpens");
		this._Prefs.RightClickOpens = this.PrefsIFace.getIntPref("settings.RightClickOpens");
		this._Prefs.ShowElements = this.PrefsIFace.getIntPref("settings.ShowElements");
	},
	loadGPrefs: function BMB_loadPrefs()
	{
		this.PrefsIFace = this.prefRootG;
		
		this._Prefs.firstRun = this.PrefsIFace.getBoolPref("firstRun");
		this._Prefs.enableDebug = this.PrefsIFace.getBoolPref("enableDebug");
	},
	
	addButtonAtFirstTime: function addButtonAtFirstTime()
	{
		//add menubutton to nav-bar on first startup and search in all custom toolbars if BMB is in there
		var navbar = document.getElementById("nav-bar");
		var toolbars = document.getElementsByTagName("toolbar");
		if (this._Prefs.firstRun)
		{
			var foundMB = false;
			var currentset = "";
			//alle toolbars nach BMB oder HMB durchsuchen
			for (let erg=-1, i=0, l=toolbars.length; i < l; ++i)
			{
				if (toolbars[i].getAttribute("class").indexOf("chromeclass") != -1)
				{
					currentset = toolbars[i].currentSet;
					erg = currentset.search(/(bookmarks|history)-menu-button/g);
					if (erg > -1)
					{
						foundMB = true;
						break;
					}
				}
			}
			toolbars = null;
			
			if (!foundMB && !navbar.getAttribute("collapsed"))
			{
				let check = confirm(this.StringBundle.getString("bmb-ext.FirstUsageAddButton"));
				if (check)
				{
					var menu = document.getElementById("bookmarksMenu");
					menu.hidden = true;
					//using insertItem works!
					let aBeforeElt = document.getElementById("urlbar-container");
					navbar.insertItem("bookmarks-menu-button", aBeforeElt);
					navbar.insertItem("history-menu-button", aBeforeElt);
					aBeforeElt = null;
					currentset = navbar.currentSet;
					navbar.setAttribute("currentset", currentset);
					document.persist("nav-bar", "currentset");
					
					//geht alle einträge durch und entfernt nur meine settings
					var menupopup = document.getElementById("bookmarksMenuPopup");
					var childnodes = menupopup.childNodes;
					for (let i=0, l=childnodes.length; i<l; i++)
					{
						if (childnodes[i].id == "bmb-settings")
						{
							menupopup.removeChild(childnodes[i]);
							break;
						}
					}
					bmbextHistory.hideHistoryMenu();		//HMB ist zum Zeitpunkt von ersten hideGoMenu() Aufruf noch nicht da
				}
			}
		}
		this._Prefs.firstRun = false;
		this.PrefsIFace = this.prefRootG;
		this.PrefsIFace.setBoolPref("firstRun", false);
	},
	
	onMouseOverButton: function BMB_onMouseOverButton(event)
	{
		if (!this.open)
		{
			this.setBehavior(true);
		}
		this.checkForContextMenu();
		this.showMenu();
	},
	/**
		remove/disable toolbar context menu, if needed (right-click perfoms user-action)
	*/
	checkForContextMenu: function BMB_checkForContextMenu()
	{
		var button = document.getElementById("bookmarks-menu-button");
		
		if (this._Prefs.RightClickOpens > 0)
		{
			button.setAttribute("context", "");
		}
		else
		{
			button.setAttribute("context", "toolbar-context-menu");
		}
	},
	showHide: function BMB_showHide()
	{
		var menu = document.getElementById("bookmarksMenu");
		var bmb = document.getElementById("bookmarks-menu-button");
		if (bmb)
		{
			this.checkForContextMenu();
			if (menu)
			{
				menu.hidden = this._Prefs.hideBM;
			}
		}
		else
		{
			//add the BMB settings to the normal bookmarks menu, if not already added
			var bmbSettings = document.getElementById("bmb-settings");
			if (menu && !menu.hidden && !bmbSettings)
			{
				bmbSettings = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menuitem");
				bmbSettings.id = "bmb-settings";
				bmbSettings.setAttribute("label", this.StringBundle.getString("bmb-ext.settingsmenu.label"));
				bmbSettings.setAttribute("oncommand", "bmbext.OpenSettings(0)");
				bmbSettings.setAttribute("class", "menuitem-iconic");
				menu.firstChild.insertBefore(bmbSettings, menu.firstChild.firstChild);
			}
			bmbSettings = null;
		}
	},
	findBMBCSSSheet: function BMB_findBMBCSSSheet()
	{
		var CSSSheets = document.styleSheets;
		for (let i=0, l=CSSSheets.length; i<l; i++)
		{
			if (CSSSheets[i].href == "chrome://bmb-ext/skin/bmb-ext.css")
			{
				this.BMBCSSSheet.globalIndex = i;
				this.BMBCSSSheet.sheetObj = CSSSheets[i];
				break;
			}
		}
	},
	showHideDropmarker: function BMB_showHideDropmarker()
	{
		var sheet = this.BMBCSSSheet;
		if (sheet.globalIndex < 0)	// was not found
			return;
		
		var lastRuleIndex = sheet.sheetObj.cssRules.length - 1;
		
		if (!sheet.dropmarkerHidden && this._Prefs.hideDropmarker)
		{
			sheet.sheetObj.insertRule("#bookmarks-menu-button .toolbarbutton-menu-dropmarker { display: none !important;}", lastRuleIndex + 1);
			sheet.dropmarkerHidden = true;
			return;
		}
		
		if (sheet.dropmarkerHidden && !this._Prefs.hideDropmarker)
		{
			var indexToRemove = -1;
			for (let i=lastRuleIndex; i>=0; i--)
			{
				if (sheet.sheetObj.cssRules[i].selectorText == "#bookmarks-menu-button .toolbarbutton-menu-dropmarker")
				{
					indexToRemove = i;
					break;
				}
			}
			
			if (indexToRemove >= 0)
			{
				sheet.sheetObj.deleteRule(indexToRemove);
				sheet.dropmarkerHidden = false;
			}
		}
	},
	showHideOptElements: function BMB_showHideOptElements()
	{
		if (document.getElementById("bookmarks-menu-button"))	// bmb gibt es beim ersten start nach der installation noch nicht
		{
			document.getElementById("bmb-bookmarksToolbarFolderMenu").previousSibling.hidden = (this._Prefs.ShowElements == 0);
			document.getElementById("bmb-bookmarksToolbarFolderMenu").hidden = ((this._Prefs.ShowElements&1) !== 1);
			document.getElementById("bmb-UnfiledBookmarksMenu").hidden = ((this._Prefs.ShowElements&2) !== 2);
			document.getElementById("bmb-TagsMenu").hidden = ((this._Prefs.ShowElements&4) !== 4);
		}
	},
	/**
		adds/removes BMB settings menuitem
	*/
	addBMBSettings: function BMB_addBMBSettings()
	{
		var settings = document.getElementById("bmb-settings");
		var settingssep = document.getElementById("bmb-settings-sep");
		settings.hidden = this._Prefs.hideBMBS;
		settingssep.hidden = this._Prefs.hideBMBS;
	},
	
	buildMenu: function BMB_buildMenu()
	{
		var menupopup = document.getElementById("bookmarksMenuPopup");		//orig. menupopup
		//remove possible added BMB settings by showHide(), when BMB was not present
		let bmbSettings = document.getElementById("bmb-settings");
		if (bmbSettings && bmbSettings.parentNode == menupopup)
		{
			menupopup.removeChild(bmbSettings);
		}
		bmbSettings = null;
		
		let item = document.getElementById("bmb-bookmarksToolbarFolderPopup");
		item.controllers.appendController(new BMBSCmdController(item));
		
		this.addBMBSettings();
		
		this._MiscVars.MenuIsThere = true;
	},
	destroyMenu: function BMB_destroyMenu()
	{
		var bmb = document.getElementById("bookmarks-menu-button");
		if (!bmb.open)
		{
			this.setBehavior(false);
			this._MiscVars.MenuIsThere = false;
		}
	},
	showMenu: function BMB_showMenu()
	{
	//this way does solve the problem that the menu is not there after the first start when the button is in the toolbarpalettebox but not in the toolbar
	//when the button is there (toolbar) and then removed from the toolbar and added again to it (without a browser restart), the menuentries are gone - only bookmarks are there.
		
		if (!this._MiscVars.MenuIsThere)
		{
			this.setBehavior(true);
			this.buildMenu();
		}
		else
		{
			this.updateSubscribeToPage();
		}
	},
	
	UpdateBookmarkAllTabsMenuitem: function BMB_UpdateBookmarkAllTabsMenuitem()
	{
		var bookmarkAllCmd = document.getElementById("bmb-bookmarkAllCmd");
		if (bookmarkAllCmd)
		{
			UpdateBookmarkAllTabsMenuitem();
		}
	},
	updateSubscribeToPage: function BMB_updateSubscribeToPage()
	{
		var feedButton = document.getElementById("feed-button");
		if (!feedButton)
			return;
		
		feedButton = null;
		var feedMenuitem = document.getElementById("bmb-subscribeToPageMenuitem");
		if (!feedMenuitem)
			return;
		
		var feedMenu = document.getElementById("bmb-subscribeToPageMenupopup");	// heißt zwar menupopup, ist aber ein menu
		if (!feedMenu)
			return;
		
		var feeds = gBrowser.mCurrentBrowser.feeds;
		if (!feeds || feeds.length == 0)
		{
			feedMenu.setAttribute("hidden", "true");
			feedMenuitem.setAttribute("disabled", "true");
			feedMenuitem.removeAttribute("hidden");
		}
		else
		{
			if (feeds.length > 1)
			{
				feedMenu.removeAttribute("hidden");
				feedMenuitem.setAttribute("hidden", "true");
			}
			else
			{
				feedMenu.setAttribute("hidden", "true");
				feedMenuitem.setAttribute("feed", feeds[0].href);
				feedMenuitem.removeAttribute("disabled");
				feedMenuitem.removeAttribute("hidden");
			}
		}
	},
	updateSubscribeToPageMenuitems: function BMB_updateSubscribeToPageMenuitems()
	{
		var feedMenu = document.getElementById("bmb-subscribeToPageMenupopup");
		var items = feedMenu.getElementsByTagName("menuitem");
		for (let i=0, l=items.length; i < l; i++)
		{
			items[i].className = "menuitem-iconic feed";
		}
	},
	
	setStatus: function BMB_setStatus(aTarget)	//sets the text of the statusbar
	{
		this.StatusBarObj.label = aTarget.statusText;
	},
	unsetStatus: function BMB_unsetStatus()	//clears the statusbar text again
	{
		this.StatusBarObj.removeAttribute("label");
	},
	setStatusAndTooltiptext: function BMB_setStatusAndTooltiptext(aEvent)
	{
		var Folder = aEvent.target;
		for (let i=0, l=Folder.childNodes.length, bookmark; i < l; i++)
		{
			bookmark = Folder.childNodes[i];
			if (bookmark.node && bookmark.node.uri != "" && bookmark.tooltipText != bookmark.label)
			{
				bookmark.tooltipText = bookmark.label;
				bookmark.setAttribute("onmouseover", "bmbext.setStatus(this); " + bookmark.getAttribute("onmouseover"));
			}
		}
	},
	
	catchMKey: function BMB_catchMKey(aEvent)
	{
		if (aEvent.currentTarget.disabled)
			return;
		
		switch (aEvent.button)
		{
			case 0:	//left button uses oncommand
				break;
				
			case 1:	//middle button
				if (aEvent.currentTarget.open)
				{
					BookmarksEventHandler.onClick(aEvent);
				}
				else
				{
					switch (this._Prefs.MiddleClickOpens)
					{
						case 0:
							break;
						case 1:
							toggleSidebar("viewBookmarksSidebar");
							break;
						case 2:
							PlacesCommandHook.showPlacesOrganizer(ORGANIZER_ROOT_BOOKMARKS);
							break;
						case 3:
							for (let i = 0, l = gBrowser.browsers.length; i < l; i++)
							{
								if (gBrowser.browsers[i].contentDocument.location == "about:bookmarks")
								{
									gBrowser.selectTabAtIndex(i);	// nur ab Fx 3.5
									return;
								}
							}
							let where = (gBrowser.browsers.length == 1 && gBrowser.browsers[0].contentDocument.location == "about:blank") ? "current" : "tab";
							openUILinkIn("about:bookmarks", where);
							break;
						default:
							break;
					}
				}
				break;
				
			case 2:	//right button
				if (!aEvent.currentTarget.open)
				{
					switch (this._Prefs.RightClickOpens)
					{
						case 0:
							break;
						case 1:
							toggleSidebar("viewBookmarksSidebar");
							break;
						case 2:
							PlacesCommandHook.showPlacesOrganizer(ORGANIZER_ROOT_BOOKMARKS);
							break;
						case 3:
							for (let i = 0, l = gBrowser.browsers.length; i < l; i++)
							{
								if (gBrowser.browsers[i].contentDocument.location == "about:bookmarks")
								{
									gBrowser.selectTabAtIndex(i);	// nur ab Fx 3.5
									return;
								}
							}
							let where = (gBrowser.browsers.length == 1 && gBrowser.browsers[0].contentDocument.location == "about:blank") ? "current" : "tab";
							openUILinkIn("about:bookmarks", where);
							break;
						default:
							break;
					}
				}	
				break;
				
			default:	//unknown
				//alert("What the heck?!\nUnknown mouse button used.\nPlease contact the author of Bookmarks Menu Button");
				break;
		}
	},
	
	openMenu: function BMB_openMenu()
	{
		this.showMenu();	//just in case the button was not hoverd before so i need to build the menu
		var bmb = document.getElementById("bookmarks-menu-button");
		bmb.firstChild.openPopup(bmb, "after_start", 0, 0, false, false);
		bmb.firstChild.setAttribute("autoopened", "true");
	},
	
	catchKKey: function BMB_catchKKey(aEvent)
	{
		//shortcut key event handler
		var bmb = document.getElementById("bookmarks-menu-button");
		if (!bmb || bmb.open)
		{
			bmbext._MiscVars.ShortcutKeyPressed = false;	//zur sicherheit
			return;
		}
		
		if (!bmbext._MiscVars.ShortcutKey)
		{
			return;
		}
		
		if (aEvent.type== "keydown" && !bmbext._MiscVars.ShortcutKeyPressed)
		{
			if (aEvent.altKey && (aEvent.keyCode == bmbext._MiscVars.ShortcutKey))
			{
				bmbext._MiscVars.ShortcutKeyPressed = true;
			}
		}
		if (aEvent.type== "keyup" && bmbext._MiscVars.ShortcutKeyPressed)
		{
			if (aEvent.altKey && (aEvent.keyCode == bmbext._MiscVars.ShortcutKey))
			{
				bmbext._MiscVars.ShortcutKeyPressed = false;
				bmbext.openMenu();
			}
		}
	},
	
	setBehavior: function BMB_setBehavior(menu)
	{
		//this function is used to switch between the two different behaviors
		//when closed you can drag and drop a link on the button to add it to your bookmarks
		//when open you can move bookmarks and folders within the menu
		var bmb = document.getElementById("bookmarks-menu-button");
		if (bmb)
		{
			if (menu)	// open - ability to move bookmarks within the menu
			{
				bmb.setAttribute("ondrop", "nsDragAndDrop.drop(event, BMBDropHandler);");
				bmb.setAttribute("ondragover", "nsDragAndDrop.dragOver(event, BMBDropHandler);");
				bmb.removeAttribute("ondragenter");
				bmb.setAttribute("ondragexit", "nsDragAndDrop.dragExit(event, BMBDropHandler);");
			}
			else		// closed - ability to drop bms on the menu to add them
			{
				bmb.setAttribute("ondrop", "bookmarksButtonObserver.onDrop(event); event.stopPropagation()");
				bmb.setAttribute("ondragover", "bookmarksButtonObserver.onDragOver(event); event.stopPropagation()");
				bmb.setAttribute("ondragenter", "BMBDropHandler.onButtonDragEnter(event);");
				bmb.setAttribute("ondragleave", "bookmarksButtonObserver.onDragLeave(event); BMBDropHandler.onDragExit(event); event.stopPropagation()");
			}
		}
	},
	
	OpenSettings: function BMB_OpenSettings(aTabIndex)
	{
		window.openDialog("chrome://bmb-ext/content/bmb-ext_prefDialog.xul", "wBMB_Settings", "chrome,centerscreen,resizable", aTabIndex ? aTabIndex : 0);
	},
	
	patchMethods: function BMB_patchMethods()
	{
		eval("BookmarksEventHandler.onPopupShowing=" + BookmarksEventHandler.onPopupShowing.toString().replace(
			'target.id != "bookmarksMenuPopup" &&',
			'target.id != "bookmarksMenuPopup" && \
			target.id != "menubutton_BookmarksPopup" &&')
		);
	}
};

/*
	BookmarksMenuDropHandler for BMB
*/
const BMBDropHandler = {
	_openBMBTimer: null,
	// öffnet den button und löscht gefeuerten timer, damit er wieder feuern kann
	_openBMB: function BMBDH__openBMB(event)
	{
		if (this._openBMBTimer)
			delete this._openBMBTimer;
		
		bmbext.openMenu();
	},
	// löst timer aus, der nach x ms den button öffnet, um ein lesezeichen gezielt ablegen zu können (wie beim normalen lesezeichen menü)
	onButtonDragEnter: function BMBDH_onButtonDragEnter(event)
	{
		// drag and drop to menus is not supported on XP_MACOSX
		if (!PlacesMenuDNDController._dragSupported || this._openBMBTimer)
			return;
		
		function Callback(callee, func, args)
		{
			this._args = args;
			this._func = func;
			this._callee = callee;
		}
		Callback.prototype = {
			notify: function(timer)
			{
				this._func.apply(this._callee, this._args);
			}
		}
		var timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
		timer.initWithCallback(new Callback(this, this._openBMB, [event]), 1000, Ci.nsITimer.TYPE_ONE_SHOT);
		this._openBMBTimer = timer;
	},
	
	getSupportedFlavours: function BMBDH_getSupportedFlavours()
	{
		var view = document.getElementById("menubutton_BookmarksPopup");
		return view.getSupportedFlavours();
	},
	canDrop: function BMBDH_canDrop(event, session)
	{
		PlacesControllerDragHelper.currentDataTransfer = event.dataTransfer;	// fx 3.5
		var ip = new InsertionPoint(PlacesUtils.bookmarksMenuFolderId, -1);
		return ip && PlacesControllerDragHelper.canDrop(ip);
	},
	onDrop: function BMBDH_onDrop(event, session) // Fx 3.6
	{
		return;
	},
	onDragOver: function BMBDH_onDragOver(event, flavor, session)
	{
		// fx 3.6
		if (!this.canDrop(event, session))
			event.dataTransfer.effectAllowed = "none";
	},
	onDragExit: function BMBDH_onDragExit(event, session)
	{
		PlacesControllerDragHelper.currentDataTransfer = null;
		
		// timer abbrechen, wenn vorher wieder vom button weg
		if (this._openBMBTimer)
		{
			this._openBMBTimer.cancel();
			delete this._openBMBTimer;
		}
	}
};

window.addEventListener("load", bmbext, false);		//extension should load on browser start-up
window.addEventListener("unload", bmbext, false);		//and unload on exit